;********************************** Indexado_02.asm *************************************
;
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
;
; Controla el nivel de un depsito de lquido. Utiliza (entre parntesis las lneas del
; microcontrolador a la que se han conectado):
; -	Tres sondas detectoras: SV, Sonda de Vaco (RA0); SLL, Sonda de LLenado (RA1);
;	SR, Sonda de Rebose (RA2).
; -	Dos bombas de agua: B1 (RB5), B2 (RB6).
; - Cinco indicadores: Vacio (RB0), Llenandose (RB1), Lleno (RB2), Rebose (RB3),
;	Alarma (RB4).
;
; Su funcionamiento: 
; -	Cuando ninguna de las sondas est mojada se entiende que el depsito est vaco y
;	se accionarn las dos bombas. El indicador "Vacio" se iluminar.
; -	Cuando el nivel del lquido toque la sonda de vaco "SV" seguir llenndose el
; 	depsito con las dos bombas. El indicador "Llenandose" se ilumina.
; -	Cuando el nivel del lquido toca la sonda de llenado "SLL", para la bomba B2, quedando
;	B1 activada en modo mantenimiento. El indicador "Lleno" se ilumina.
; -	Si el nivel del lquido moja la sonda de rebose "SR" se apaga tambin la bomba B1,
;	quedando las dos bombas fuera de servicio. El indicador "Rebose" se enciende.
; -	Cuando se produce un fallo o mal funcionamiento en las sondas de entrada (por
;	ejemplo que se active la sonda de rebose y no active la de vaco) se paran
;	las dos bombas. El indicador "Alarma" se ilumina.
;
; Segn el enunciado del problema, teniendo en cuenta las conexiones citadas y poniendo la
; salida no utilizada (RB7) siempre a cero, la tabla de verdad resultante es:
;
; RA2.. RA0 | RB7 ...          ... RB0
; ------------|--------------------------------
;  0   0   0  |  0   1   1   0   0   0   0   1	(Configuracin 0. Estado "Vacio").
;  0   0   1  |  0   1   1   0   0   0   1   0	(Configuracin 1. Estado "Llenandose").
;  0   1   0  |  0   0   0   1   0   0   0   0	(Configuracin 2. Estado "Alarma").
;  0   1   1  |  0   0   1   0   0   1   0   0	(Configuracin 3. Estado "Lleno").
;  1   0   0  |  0   0   0   1   0   0   0   0	(Configuracin 4. Estado "Alarma").
;  1   0   1  |  0   0   0   1   0   0   0   0	(Configuracin 5. Estado "Alarma").
;  1   1   0  |  0   0   0   1   0   0   0   0	(Configuracin 6. Estado "Alarma").
;  1   1   1  |  0   0   0   0   1   0   0   0	(Configuracin 7. Estado "Rebose").
;
; ZONA DE DATOS **********************************************************************

	LIST		P=16F84A
	INCLUDE		<P16F84A.INC>
	__CONFIG	_CP_OFF &  _WDT_OFF & _PWRTE_ON & _XT_OSC

; ZONA DE CDIGOS ********************************************************************

	ORG 	0					; El programa comienza en la direccin 0.
Inicio
	clrf	PORTB				; Debe estar a cero cuando el puerto se configure como salida.
	bsf		STATUS,RP0			; Acceso al Banco 1.
	clrf	TRISB				; Las lneas del Puerto B se configuran como salida.
	movlw	b'00011111'			; Las 5 lneas del Puerto A se configuran como entrada.
	movwf	TRISA
	bcf		STATUS,RP0			; Acceso al Banco 0.
Principal
	movf	PORTA,W				; Lee los sensores.
	andlw	b'00000111'			; Mscara para quedarse con el valor de los sensores.
	addwf	PCL,F				; Salta a la configuracin adecuada.
	goto	Configuracion0
	goto	Configuracion1
	goto	Configuracion2
	goto	Configuracion3
	goto	Configuracion4
	goto	Configuracion5
	goto	Configuracion6
	goto	Configuracion7
Configuracion0
	movlw 	b'01100001'			; Estado "Vacio" (configuracin 0).
	goto	ActivaSalida
Configuracion1
	movlw 	b'01100010'			; Estado "Llenndose" (configuracin 1).
	goto	ActivaSalida
Configuracion2
	movlw 	b'00010000'			; Estado "Alarma" (configuracin 2).
	goto	ActivaSalida
Configuracion3
	movlw 	b'00100100'			; Estado "Lleno" (configuracin 3).
	goto	ActivaSalida
Configuracion4
	movlw 	b'00010000'			; Estado "Alarma" (configuracin 4).
	goto	ActivaSalida
Configuracion5
	movlw 	b'00010000'			; Estado "Alarma" (configuracin 5).
	goto	ActivaSalida
Configuracion6
	movlw 	b'00010000'			; Estado "Alarma" (configuracin 6).
	goto	ActivaSalida
Configuracion7
	movlw 	b'00001000'			; Estado "Rebose" (configuracin 7).
ActivaSalida
	movwf	PORTB				; Visualiza por el puerto de salida.
	goto 	Principal

	END
	
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
