;*********************************** Indirecto_02.asm ***********************************
;
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
;
; Este programa comprueba el funcionamiento de la lectura y escritura en la memoria de 
; datos mediante direccionamiento indirecto.
;
; Se trata de escribir, a partir de la ltima direccin ocupada de memoria RAM de datos
; hasta el final, el contenido con el valor de la direccin. As, por ejemplo, en la
; direccin 20h se escribe "20", en la direccin 21h se escribe "21", en la direccin 22h
; se escribe "22" y as sucesivamente. Se escribir hasta la direccin 4Fh que son las
; direcciones implementadas en PIC16F84A.
;
; A continuacin se proceder a la lectura de la memoria RAM de datos completa, desde la
; la direccin 00h hasta la 4Fh. En la pantalla se visualizar la direccin y su
; contenido. Observar que direcciones de 00h a 0Bh corresponden al SFR. Cada
; visualizacin se mantendr durante medio segundo en pantalla.
;
; En los contadores se carga siempre un valor ms, porque la instruccin
; "decfsz RAM_Contador,F" salta cuando es cero y no permite visualizar el contenido del
; ltimo registro.
 
; ZONA DE DATOS **********************************************************************

	LIST		P=16F84A
	INCLUDE		<P16F84A.INC>
	__CONFIG	_CP_OFF &  _WDT_OFF & _PWRTE_ON & _XT_OSC

	CBLOCK	0x0C
	RAM_Dato						; Dato a escribir o leer.
	RAM_Contador
	ENDC

RAM_ULTIMA_DIR	EQU	4Fh				; Ultima direccin utilizada.

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
Inicio
	call	LCD_Inicializa
	movlw	RAM_PrimeraLibre		; Este es la primer dato a escribir en la RAM.
	movwf	RAM_Dato
	movlw	RAM_ULTIMA_DIR-RAM_PrimeraLibre+1 ; Nmero de posiciones a escribir.
	movwf	RAM_Contador
	movlw	RAM_PrimeraLibre
	movwf	FSR						; Primera direccin de memoria RAM a escribir
RAM_Escribe
	movf	RAM_Dato,W				; Escribe el contenido de RAM_Dato en posicin
	movwf	INDF					; apuntada por FSR.
	incf	FSR,F					; Apunta a la siguiente direccin de memoria.
	incf	RAM_Dato,F				; Incrementa el dato a cargar.
	decfsz	RAM_Contador,F
	goto	RAM_Escribe
;
; El programa principal procede a leer la RAM desde la direccin 0.
;
Principal
	movlw	RAM_ULTIMA_DIR+1 		; Nmero de posiciones a leer.
	movwf	RAM_Contador
	movlw	0						; Primera direccin a leer.
	movwf	FSR
RAM_Lee
	movf	INDF,W					; Lee el contenido de RAM_Dato en posicin
	movwf	RAM_Dato				; apuntada por FSR.
	call	VisualizaRAM
	incf	FSR,F					; Apunta a la siguiente direccin de memoria.
	decfsz	RAM_Contador,F			; Si no ha llegado al final pasa a leer la
	goto	RAM_Lee					; siguiente.
	goto	Principal
;
; Subrutina "VisualizaRAM" --------------------------------------------------------------
;
VisualizaRAM
	call	LCD_Linea1				; Pasa a visualizarla.
	movlw	MensajeDireccion
	call	LCD_Mensaje
	movf	FSR,W					; Visualiza el nmero de la posicin
	call	LCD_ByteCompleto
	call	LCD_Linea2
	movlw	MensajeContenido
	call	LCD_Mensaje
	movf	RAM_Dato,W				; Visualiza el contenido de la posicin
	call	LCD_ByteCompleto
	call	Retardo_500ms
	return

; "Mensajes" ----------------------------------------------------------------------------
;
Mensajes
	addwf	PCL,F
MensajeDireccion
	DT "Direccion: ", 0x00
MensajeContenido
	DT "Contenido: ", 0x00

	INCLUDE  <RETARDOS.INC>		; Estos includes tambin reservan posiciones de
	INCLUDE  <LCD_4BIT.INC>		; memoria RAM. Por tanto la ltima posicin habr 
	INCLUDE  <LCD_MENS.INC>		; que definirla despus de stos.

	CBLOCK
	RAM_PrimeraLibre			; Ultima posicin de RAM ocupada por una variable.
	ENDC

	END

;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
