;*********************************** Int_INT_04B.asm **********************************
;
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
;
; Cada vez que presiona el pulsador conectado a la lnea RB0/INT conmutar el estado de
; un LED conectado a la lnea RB1. La complementacin se realiza con ayuda de una funcin XOR.
;
; ZONA DE DATOS **********************************************************************

	LIST		P=16F84A
	INCLUDE		<P16F84A.INC>
	__CONFIG	_CP_OFF &  _WDT_OFF & _PWRTE_ON & _XT_OSC

	CBLOCK  0x0C
	ENDC

#DEFINE  Pulsador	PORTB,0			; Lnea donde se conecta el pulsador.
#DEFINE  LED		PORTB,1			; Lnea donde se conecta el LED.

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
	goto	Inicio
	ORG	4							; Vector de interrupcin
	goto	ServicioInterrupcion
Inicio
	bsf		STATUS,RP0				; Acceso al Banco 1.
	bsf		Pulsador				; La lnea RB0/INT se configura como entrada.
	bcf		LED						; Se configura como salida.
	bcf		OPTION_REG,NOT_RBPU		; Activa las resistencias de Pull-Up del Puerto B.
	bcf		OPTION_REG,INTEDG		; Interrupcin INT activa por flanco de bajada.
	bcf		STATUS,RP0				; Acceso al Banco 0.
	movlw	b'10010000'				; Habilita la interrupcin INT y la general.
	movwf	INTCON
Principal
	sleep							; Pasa a modo bajo consumo y espera interrupciones
	goto	Principal

; Subrutina "ServicioInterrupcion" ------------------------------------------------------
;
; Subrutina de atencin a la interrupcin. Conmuta el estado del LED. La complementacin se
; realiza con una funcin XOR. Observar que es mucho ms eficaz que la solucin anterior.

ServicioInterrupcion
	call	Retardo_20ms			; Para salvaguardarse de los rebotes.
	btfsc	Pulsador				; Comprueba si es un rebote.
	goto	FinInterrupcion			; Era un rebote y por tanto sale.
	movlw	b'00000010'				; Se hace una XOR para complementar.
	xorwf	PORTB,F
FinInterrupcion
	bcf		INTCON,INTF				; Limpia flag de reconocimiento de la interrupcin.
	retfie							; Retorna y rehabilita las interrupciones.

	INCLUDE   <RETARDOS.INC>
	END
	
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
