;**************************** Librera "M24LC256.INC" ***********************************
;
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
;
; Estas subrutinas permiten realizar las tareas de manejo de la memoria EEPROM serie
; 24LC256 que transmite y recibe la informacin va serie a travs de un bus I2C.
;
; Subrutina "M24LC256_InicializaEscritura" ----------------------------------------------
;
; Prepara la memoria para iniciar su escritura a partir de la posicin de memoria fijada
; por los registros:
; - (M24LC256_AddressHigh), indica el nmero del bloque o pgina de memoria a escribir.
; - (M24LC256_AddressLow), indica posicin a escribir dentro del bloque.

	CBLOCK
	M24LC256_AddressHigh			; Guarda el valor de la direccin de memoria a 
	M24LC256_AddressLow				; escribir o leer.
	M24LC256_Dato
	ENDC
;
M24LC256_DIR_ESCRITURA	 EQU	b'10100000'; Direccin de la memoria 24LC256 para
M24LC256_DIR_LECTURA	 EQU	b'10100001'; escritura y lectura respectivamente.

M24LC256_InicializaEscritura
	call	I2C_EnviaStart				; Enva condicin de Start.
	movlw	M24LC256_DIR_ESCRITURA		; Enva direccin de escritura del 
	call	I2C_EnviaByte				; esclavo.
	movf	M24LC256_AddressHigh,W		; A partir de la direccin apuntada por los 
	call	I2C_EnviaByte				; registros M24LC256_AddressHigh y
	movf	M24LC256_AddressLow,W		; M24LC256_AddressLow.
	call	I2C_EnviaByte
	return	

; Subrutina "M24LC256_InicializaLectura" ------------------------------------------------
;
; Prepara la memoria para iniciar su lectura a partir de la posicin de memoria fijada
; por los registros:
; - (M24LC256_AddressHigh), indica el nmero del bloque o pgina de memoria a leer.
; - (M24LC256_AddressLow), indica posicin a escribir dentro del bloque.

M24LC256_InicializaLectura
	bcf		I2C_UltimoByteLeer			; Todava no ha comenzado a leer ningn dato.
	call	I2C_EnviaStart				; Enva condicin de Start.
	movlw	M24LC256_DIR_ESCRITURA		; Enva direccin de escritura del 
	call	I2C_EnviaByte				; esclavo.
	movf	M24LC256_AddressHigh,W		; A partir de la direccin apuntada por los 
	call	I2C_EnviaByte				; registros M24LC256_AddressHigh y
	movf	M24LC256_AddressLow,W		; M24LC256_AddressLow.
	call	I2C_EnviaByte
	call	I2C_EnviaStop
;
	call	I2C_EnviaStart				; Enva condicin de Start.
	movlw	M24LC256_DIR_LECTURA		; Indica a la memoria 24LC256 que va a 
	call	I2C_EnviaByte				; proceder a la lectura.
	return

; Subrutina "M24LC256_Mensaje_a_LCD" ----------------------------------------------------
;
; Lee el mensaje grabado en la memoria 24LC256 y lo visualiza en la pantalla del mdulo LCD.
; En el registro de trabajo W se introduce la pgina de la memoria a partir de cuya primera
; posicin se va a leer. La lectura termina cuando encuentre el cdigo 0x00. As por ejemplo,
; si (W)=2Fh lee el mensaje que comienza en la posicin 0 de la pgina 2Fh de la memoria,
; que es la posicin 2F00h absoluta. 
; 
	CBLOCK
;	M24LC256_ValorCaracter				; Valor ASCII del carcter ledo.
	ENDC

;M24LC256_Mensaje_a_LCD
	movwf	M24LC256_AddressHigh		; Apunta al inicio de la pgina correspondiente.
	clrf	M24LC256_AddressLow
	call	M24LC256_InicializaLectura
M24LC256_LeeOtroByte
	call	I2C_LeeByte					; Lee la memoria 24LC256.
	movwf	M24LC256_ValorCaracter		; Guarda el valor de carcter.
	movf	M24LC256_ValorCaracter,F	; Lo nico que hace es posicionar flag Z. En caso
	btfsc	STATUS,Z					; que sea "0x00", que es cdigo indicador final	
	goto	M24LC256_FinMensaje			; del mensaje, sale de la subrutina.
	movf	M24LC256_ValorCaracter,W	; Recupera el valor ledo.
	call	LCD_Caracter				; Lo visualiza en la pantalla del LCD.
	incf	M24LC256_AddressLow,F		; Apunta a la siguiente posicin.
	goto	M24LC256_LeeOtroByte
;M24LC256_FinMensaje
;	call	M24LC_256_FinalizaLectura
;	return

; Subrutina "M24LC256_FinalizaLectura" ------------------------------------------------
;
; Activa el bit I2C_UltimoByteLeer para que la subrutina I2C_LeeByte ponga en alta
; impedancia la lnea SDA y pueda ejecutarse posteriormente la condicin de Start o Stop
; que fija el protocolo del bus I2C.

M24LC256_FinalizaLectura
	bsf	I2C_UltimoByteLeer				; Con estas dos instrucciones se pone en 
	call	I2C_LeeByte				 	; alta impedancia la lnea SDA. No importa el
	call	I2C_EnviaStop				; resultado de la lectura realizada.
	return

; Subrutina "M24LC256_Mensaje_a_LCD" ----------------------------------------------------
;
; Lee un mensaje grabado en la memoria 24LC256 y lo visualiza por el mdulo LCD. En caso que
; sea ms largo que la longitud de la pantalla se desplaza hacia la izquierda con sensacin
; de movimiento. En el registro de trabajo W se introduce la pgina de la memoria a partir
; de cuya primera posicin va a leer. La visualizacin termina cuando encuentre el cdigo
; 0x00. As por ejemplo si (W)= 2Fh lee el mensaje que comienza en la posicin 0 de la
; pgina 2Fh de la memoria, que es la posicin 2F00h absoluta. 
; 
	CBLOCK
	M24LC256_ValorCaracter				; Valor ASCII del carcter ledo.
	M24LC256_CursorPosicion
	ENDC
	
M24LC256_Mensaje_a_LCD
	bcf	I2C_UltimoByteLeer
	movwf	M24LC256_AddressHigh		; Apunta al inicio de la pgina correspondiente.
	clrf	M24LC256_AddressLow
	call	M24LC256_InicializaLectura
M24LC256_PrimeraPosicion
	clrf	M24LC256_CursorPosicion		; El cursor en la posicin 0 de la lnea.
	call	LCD_Borra					; Se sita en la primera posicin de la lnea 1 y
M24LC256_VisualizaCaracter				; borra la pantalla.
	movlw	LCD_CaracteresPorLinea		; Ha llegado a final de lnea?
	subwf	M24LC256_CursorPosicion,W
	btfss	STATUS,C
	goto	M24LC256_NoEsFinalLinea
M24LC256_EsFinalLinea
	call	Retardo_200ms				; Lo mantiene visualizado durante este tiempo.
	call	Retardo_200ms
	call	M24LC256_FinalizaLectura
	incf	M24LC256_AddressLow,F
	call	M24LC256_InicializaLectura
	goto	M24LC256_PrimeraPosicion
M24LC256_NoEsFinalLinea
	call	I2C_LeeByte					; Obtiene el ASCII del carcter apuntado.
	movwf	M24LC256_ValorCaracter		; Guarda el valor de carcter.
	movf	M24LC256_ValorCaracter,F	; Lo nico que hace es posicionar flag Z. En caso
	btfsc	STATUS,Z					; que sea "0x00", que es cdigo indicador final
	goto	M24LC256_FinMensaje			; de mensaje, sale fuera.
M24LC256_NoUltimoCaracter
	call	LCD_Caracter				; Visualiza el carcter ASCII ledo.
	incf	M24LC256_CursorPosicion,F	; Contabiliza el incremento de posicin del 
										; cursor en la pantalla.
	goto	M24LC256_VisualizaCaracter	; Vuelve a visualizar el siguiente carcter
M24LC256_FinMensaje						; de la lnea.
	call	M24LC256_FinalizaLectura
	return

;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
